@@ -101,8 +101,9 @@ class Game(object):
     .. attribute:: grab_input
 
        Whether or not all input should be forcibly grabbed by the game.
-       If this is :const:`True` and the mouse cursor is invisible, the
-       mouse will enter relative mode.
+       If this is :const:`True` and :attr:`sge.mouse.visible` is
+       :const:`False`, the mouse will be in relative mode.  Otherwise,
+       the mouse will be in absolute mode.
 
     .. attribute:: window_text
 
@@ -187,14 +188,12 @@ class Game(object):
 
     .. attribute:: mouse
 
-       A :class:`sge.StellarClass` object which represents the mouse
-       cursor.  Its :attr:`sge.StellarClass.id` attribute is ``"mouse"``
-       and its bounding box is a one-pixel square.  Speed variables are
-       determined by averaging all mouse movement during the last
-       quarter of a second.  Assigning to its
-       :attr:`sge.StellarClass.visible` attribute controls whether or
-       not the mouse cursor is shown.  Setting its sprite sets the mouse
-       cursor to that sprite.
+       A :class:`sge.Object` object which represents the mouse
+       cursor.  Its :attr:`sge.Object.id` attribute is ``"mouse"``
+       and its bounding box is a one-pixel square.
+
+       Some of this object's attributes control properties of the mouse.
+       See the documentation for :mod:`sge.mouse` for more information.
 
     """
 
@@ -346,18 +345,16 @@ class Game(object):
         self.rooms = []
         self.current_room = None
 
+        self._display_surface = pygame.Surface((self.width, self.height))
         self._set_mode()
 
-        self._background_changed = False
         self._colliders = []
         self._music = None
         self._music_queue = []
         self._running = False
         self._clock = pygame.time.Clock()
-        self._pygame_sprites = pygame.sprite.LayeredDirty()
         self._numviews = 0
-        self._background = None
-        self._cleanup_rects = []
+        self._room_projections = []
         self._window_projections = []
         self.mouse = sge.Mouse()
 
@@ -449,7 +446,6 @@ class Game(object):
             self.rooms[0].start()
         else:
             self._running = True
-            self._background_changed = True
             self.event_game_start()
 
             # Store the initial state of objects
@@ -478,8 +474,6 @@ class Game(object):
                 self._object_start_collision_ellipse[obj.id] = obj.collision_ellipse
                 self._object_start_collision_precise[obj.id] = obj.collision_precise
 
-            self._display_surface = self._window.copy()
-
             self.rooms[0].start()
             self._clock.tick()
 
@@ -907,9 +901,6 @@ class Game(object):
                     input_event = sge.input.KeyRelease(k)
                     self.input_events.append(input_event)
             elif event.type == pygame.MOUSEMOTION:
-                mx, my = event.pos
-                self.mouse.mouse_x = mx - self._x
-                self.mouse.mouse_y = my - self._y
                 input_event = sge.input.MouseMove(*event.rel)
                 self.input_events.append(input_event)
             elif event.type == pygame.MOUSEBUTTONDOWN:
@@ -973,7 +964,6 @@ class Game(object):
             elif event.type == pygame.QUIT:
                 self.input_events.append(sge.input.QuitRequest())
             elif event.type == pygame.VIDEORESIZE:
-                self._window.blit(self._display_surface, (0, 0))
                 self._window_width = event.w
                 self._window_height = event.h
                 self._set_mode()
@@ -1024,60 +1014,107 @@ class Game(object):
         # Check for changes in views
         if self._numviews != len(self.current_room.views):
             self._numviews = len(self.current_room.views)
-            self._background_changed = True
 
-        # Redraw
-        w = max(1, self._display_surface.get_width())
-        h = max(1, self._display_surface.get_height())
-        if self._background_changed or self._background is None:
-            self._background = pygame.Surface((w, h))
-            b = self.current_room.background._get_background()
-            self._background.blit(b, (self._x, self._y))
-            self._display_surface.blit(self._background, (0, 0))
-            self._background_changed = False
-            self._pygame_sprites.clear(self._display_surface, self._background)
-            for sprite in self._pygame_sprites:
-                sprite.rect = pygame.Rect(0, 0, 1, 1)
-                sprite.image = pygame.Surface((1, 1))
-                sprite.image.set_colorkey((0, 0, 0))
-            self._pygame_sprites.update()
-            self._pygame_sprites.draw(self._display_surface)
-            dirty = [self._display_surface.get_rect()]
-        else:
-            self._pygame_sprites.clear(self._display_surface, self._background)
-            self._pygame_sprites.update()
-            dirty = self._pygame_sprites.draw(self._display_surface)
+        # Clear display surface
+        self._display_surface.fill((0, 0, 0))
 
-        self._window.blit(self._display_surface, (0, 0))
+        # Draw views
+        for view in self.current_room.views:
+            view_surf = pygame.Surface((view.width, view.height))
+            view_surf.fill(pygame.Color(*self.current_room.background.color))
+
+            images = []
+
+            for layer in self.current_room.background.layers:
+                img = layer._get_image()
+                x = (sge.game.current_room.background_x + layer.x -
+                     (view.x * layer.xscroll_rate))
+                y = (sge.game.current_room.background_y + layer.y -
+                     (view.y * layer.yscroll_rate))
+                img_w = max(1, img.get_width())
+                img_h = max(1, img.get_height())
+
+                # These equations bring the position to the largest
+                # values possible while still being less than the
+                # location we're getting the surface at.  This is to
+                # minimize the number of repeat blittings.
+                if layer.xrepeat:
+                    x = (x % img_w) - img_w
+                if layer.yrepeat:
+                    y = (y % img_h) - img_h
+
+                # Apply the origin so the positions are as expected.
+                x -= layer.sprite.origin_x
+                y -= layer.sprite.origin_y
+
+                # Return x and y to being room relatives instead of view
+                # relatives.
+                x += view.x
+                y += view.y
+
+                xstart = x
+                ystart = y
+
+                if layer.xrepeat and layer.yrepeat:
+                    for y in range(ystart, view.y + view.height, img_h):
+                        for x in range(xstart, view.x + view.width, img_w):
+                            images.append((img, x, y, layer.z, None))
+                elif (layer.xrepeat and y < view.y + view.height and
+                      y + img_h > view.y):
+                    for x in range(xstart, view.x + view.width, img_w):
+                        images.append((img, x, y, layer.z, None))
+                elif (layer.yrepeat and x < view.x + view.width and
+                      x + img_w > view.x):
+                    for y in range(ystart, view.y + view.height, img_h):
+                        images.append((img, x, y, layer.z, None))
+                elif (x < view.x + view.width and x + img_w > view.x and
+                      y < view.y + view.height and y + img_h > view.y):
+                    images.append((img, x, y, layer.z, None))
+
+            for obj in self.current_room.objects:
+                if obj is not self.mouse and obj.sprite is not None:
+                    img = obj.sprite._get_image(
+                        obj.image_index, obj.image_xscale, obj.image_yscale,
+                        obj.image_rotation, obj.image_alpha, obj.image_blend)
+                    w = obj._get_image_width()
+                    nw = obj._get_normal_image_width()
+                    h = obj._get_image_height()
+                    nh = obj._get_normal_image_height()
+                    xoff = (w - nw) / 2
+                    yoff = (h - nh) / 2
+                    origin_x = obj.image_origin_x + xoff
+                    origin_y = obj.image_origin_y + yoff 
+                    images.append((img, obj.x - origin_x, obj.y - origin_y,
+                                   obj.z, None))
+
+            images.extend(self.current_room._projections)
+            self.current_room._projections = []
+
+            images.sort(key=lambda img: img[3])
+
+            for img in images:
+                view_surf.blit(img[0], (img[1] - view.x, img[2] - view.y),
+                               None, sge._get_blend_flags(img[4]))
+
+            self._display_surface.blit(
+                sge._scale(view_surf, view.wport, view.hport),
+                (view.xport, view.yport))
 
         # Window projections
         self.mouse.project_cursor()
-        dirty.extend(self._cleanup_rects)
-        self._cleanup_rects = self._show_projections()
-        dirty.extend(self._cleanup_rects)
+        for projection in self._window_projections:
+            (image, x, y, blend_mode) = projection
+            flags = sge._get_blend_flags(blend_mode)
+            self._display_surface.blit(image, (x, y), None, flags)
 
-        # Letterbox/pillarbox
-        top_bar = pygame.Rect(0, 0, w, self._y)
-        bottom_bar = pygame.Rect(0, h - self._y, w, self._y)
-        left_bar = pygame.Rect(0, 0, self._x, h)
-        right_bar = pygame.Rect(w - self._x, 0, self._x, h)
-        if top_bar.h > 0:
-            self._window.fill((0, 0, 0), top_bar)
-            dirty.append(top_bar)
-        if bottom_bar.h > 0:
-            self._window.fill((0, 0, 0), bottom_bar)
-            dirty.append(bottom_bar)
-        if left_bar.w > 0:
-            self._window.fill((0, 0, 0), left_bar)
-            dirty.append(left_bar)
-        if right_bar.w > 0:
-            self._window.fill((0, 0, 0), right_bar)
-            dirty.append(right_bar)
+        self._window_projections = []
 
-        if sge.hardware_rendering:
-            pygame.display.flip()
-        else:
-            pygame.display.update(dirty)
+        # Scale/blit display surface
+        self._window.blit(
+            sge._scale(self._display_surface, self.width * self._xscale,
+                       self.height * self._yscale), (self._x, self._y))
+
+        pygame.display.flip()
 
     def register_class(self, cls):
         """Register a class with the SGE.
@@ -1711,6 +1748,8 @@ class Game(object):
     def _set_mode(self):
         # Set the mode of the screen based on self.width, self.height,
         # and self.fullscreen.
+        self._display_surface = sge._scale(self._display_surface, self.width,
+                                           self.height)
         info = pygame.display.Info()
 
         if self.scale:
@@ -1718,26 +1757,21 @@ class Game(object):
             self._yscale = self.scale
 
         if self.fullscreen or not info.wm:
-            flags = pygame.FULLSCREEN
-            if sge.hardware_rendering:
-                flags |= pygame.HWSURFACE | pygame.DOUBLEBUF
+            flags = pygame.FULLSCREEN | pygame.HWSURFACE | pygame.DOUBLEBUF
 
             modes = pygame.display.list_modes()
             if modes != -1 and modes:
                 self._window_width, self._window_height = modes[0]
             else:
-                w = "Couldn't find out the maximum resolution! Assuming 640x480."
+                w = "Couldn't find out the maximum resolution! Assuming 1024x768."
                 warnings.warn(w)
-                self._window_width = 640
-                self._window_height = 480
+                self._window_width = 1024
+                self._window_height = 768
 
-            self._background_changed = True
             self._window = pygame.display.set_mode(
                 (self._window_width, self._window_height), flags)
 
             if not self.scale:
-                ##self._xscale = info.current_w / self.width
-                ##self._yscale = info.current_h / self.height
                 self._xscale = self._window_width / self.width
                 self._yscale = self._window_height / self.height
 
@@ -1752,7 +1786,7 @@ class Game(object):
         else:
             self._x = 0
             self._y = 0
-            flags = 0
+            flags = pygame.HWSURFACE | pygame.DOUBLEBUF
 
             # Decide window size
             if self.scale:
@@ -1768,9 +1802,6 @@ class Game(object):
 
                 flags |= pygame.RESIZABLE
 
-            if sge.hardware_rendering:
-                flags |= pygame.HWSURFACE | pygame.DOUBLEBUF
-
             self._window = pygame.display.set_mode(
                 (self._window_width, self._window_height), flags)
 
@@ -1779,9 +1810,6 @@ class Game(object):
             self._x = int(round((w - self.width * self._xscale) / 2))
             self._y = int(round((h - self.height * self._yscale) / 2))
 
-        self._display_surface = self._window.copy()
-        self._background_changed = True
-
     def _get_dot_sprite(self, color):
         # Return a sprite for the given dot.
         i = tuple(color)
@@ -1899,19 +1927,6 @@ class Game(object):
 
         return sprite
 
-    def _show_projections(self):
-        # Show the window projections and return the area rects list.
-        rects = []
-        for projection in self._window_projections:
-            (image, x, y, blend_mode) = projection
-            rect = image.get_rect(left=(self._x + x * self._xscale),
-                                  top=(self._y + y * self._yscale))
-            self._window.blit(image, rect)
-            rects.append(rect)
-
-        self._window_projections = []
-        return rects
-
     def _get_channel(self):
         # Return a channel for a sound effect to use.
         assert pygame.mixer.get_init()
