@@ -783,22 +783,7 @@ class Sprite(object):
         y -= int(round(sprite.origin_y))
         image %= len(sprite._baseimages)
 
-        if blend_mode is not None and blend_mode & sge.BLEND_SCREEN:
-            w = "Screen blend mode not supported. Normal blending used instead."
-            warnings.warn(w)
-
-        pygame_flags = {
-            sge.BLEND_RGBA_ADD: pygame.BLEND_RGBA_ADD,
-            sge.BLEND_RGBA_SUBTRACT: pygame.BLEND_RGBA_SUB,
-            sge.BLEND_RGBA_MULTIPLY: pygame.BLEND_RGBA_MULT,
-            sge.BLEND_RGBA_MINIMUM: pygame.BLEND_RGBA_MIN,
-            sge.BLEND_RGBA_MAXIMUM: pygame.BLEND_RGBA_MAX,
-            sge.BLEND_RGB_ADD: pygame.BLEND_RGB_ADD,
-            sge.BLEND_RGB_SUBTRACT: pygame.BLEND_RGB_SUB,
-            sge.BLEND_RGB_MULTIPLY: pygame.BLEND_RGB_MULT,
-            sge.BLEND_RGB_MINIMUM: pygame.BLEND_RGB_MIN,
-            sge.BLEND_RGB_MAXIMUM: pygame.BLEND_RGB_MAX
-            }.setdefault(blend_mode, 0)
+        pygame_flags = sge._get_blend_flags(blend_mode)
 
         for i in range(self.frames):
             if frame is None or frame % self.frames == i:
@@ -1138,30 +1123,13 @@ class Sprite(object):
             sge.Sprite.from_screenshot().save("foo.png")
 
         """
-        window = pygame.display.get_surface()
-        w = sge.game.width * sge.game._xscale
-        h = sge.game.height * sge.game._yscale
-        display_surf = pygame.Surface((w, h))
-        display_surf.blit(window, (-sge.game._x, -sge.game._y))
-
-        if sge.game.scale_smooth:
-            try:
-                display_surf = pygame.transform.smoothscale(
-                    display_surf, (sge.game.width, sge.game.height))
-            except pygame.error:
-                display_surf = pygame.transform.scale(
-                    display_surf, (sge.game.width, sge.game.height))
-        else:
-            display_surf = pygame.transform.scale(
-                display_surf, (sge.game.width, sge.game.height))
-
         if width is None:
-            width = display_surf.get_width() - x
+            width = sge.game.width - x
         if height is None:
-            height = display_surf.get_height() - y
+            height = sge.game.height - y
 
         sprite = cls(width=width, height=height)
-        sprite._baseimages[0].blit(display_surf, (-x, -y))
+        sprite._baseimages[0].blit(sge.game._display_surface, (-x, -y))
         sprite._refresh()
         return sprite
 
@@ -1188,7 +1156,6 @@ class Sprite(object):
     def _refresh(self):
         # Set the _images list based on the variables.
         if not self._locked:
-            sge.game._background_changed = True
             self._images = []
             for image in self._baseimages:
                 img = self._set_transparency(image)
@@ -1213,8 +1180,8 @@ class Sprite(object):
                    blend=None):
         num %= len(self._images)
         # Return the properly sized surface.
-        t = (sge.game._xscale, sge.game._yscale, xscale, yscale, rotation,
-             alpha, tuple(blend) if blend is not None else blend)
+        t = (xscale, yscale, rotation, alpha,
+             tuple(blend) if blend is not None else blend)
         if t in self._images[num]:
             return self._images[num][t]
         else:
@@ -1225,7 +1192,7 @@ class Sprite(object):
                 yflip = yscale < 0
                 img = pygame.transform.flip(img, xflip, yflip)
                 img = sge._scale(img, self.width * abs(xscale),
-                             self.height * abs(yscale))
+                                 self.height * abs(yscale))
 
                 if rotation != 0:
                     img = pygame.transform.rotate(img, rotation)
